import 'package:ecg_list_view/rpc/rpc_bridge.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:get/get.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'index.dart';
import 'widgets/build_record_view.dart';
import 'widgets/create_exam.dart';
import 'widgets/filter_time.dart';
import 'widgets/icon_tab_list.dart';
import 'widgets/last_record.dart';
import 'widgets/no_data_view.dart';
import 'widgets/search_input.dart';
import 'widgets/table/table.dart';
import 'widgets/table/table_column.dart';
import 'package:fis_ui/index.dart';
import 'package:intl/intl.dart';
import 'widgets/table_pagination.dart';

class EcgListPage extends GetView<EcgListController> {
  ///reportCode&ElectrocardiogramRecord,report为空则是新建报告
  final void Function(String, ElectrocardiogramRecord) onEditReport;
  final void Function(String, String) onExam;
  final bool isExistSubordinate;

  const EcgListPage({
    required this.onEditReport,
    required this.onExam,
    this.isExistSubordinate = false,
    Key? key,
  }) : super(key: key);

  // 主视图
  Widget _buildView(BuildContext context) {
    return Row(
      children: [
        Expanded(
          child: Column(
            children: [
              Container(
                margin: const EdgeInsets.all(5),
                height: 50,
                child: _buildHeader(context),
              ),
              Expanded(child: _buildListView())
            ],
          ),
        ),
        Container(
          width: 300,
          decoration: BoxDecoration(
            border: Border(
              left: BorderSide(
                color: Colors.grey[300]!,
                width: 1,
              ),
            ),
          ),
          child: _buildRightContent(),
        )
      ],
    );
  }

  @override
  Widget build(BuildContext context) {
    return GetBuilder<EcgListController>(
      init: EcgListController(),
      id: "ecg_list",
      builder: (_) {
        return Scaffold(
          resizeToAvoidBottomInset: false, // 防止内容自动调整以避免被键盘遮挡
          body: SafeArea(
            child: _buildView(context),
          ),
        );
      },
    );
  }

  Widget _buildHeader(BuildContext context) {
    return Row(
      children: [
        Expanded(
          child: SizedBox(
            height: 50,
            child: SearchInput(
              placeholder: "请输入关键字",
              onSearch: (v) {
                controller.getDatas(pageIndex: 1, keyword: v);
              },
            ),
          ),
        ),
        const SizedBox(
          width: 10,
        ),
        IconButton(
          onPressed: () async {
            await Get.dialog(
              FilterView(
                isExistSubordinate: isExistSubordinate,
                onConfirm: (start, end) {
                  controller.startTime = start;
                  controller.endTime = end;
                  controller.getDatas();
                },
                startTime: controller.startTime,
                endTime: controller.endTime,
              ),
            );
          },
          icon: const Icon(
            Icons.filter_alt,
            size: 24,
            color: Colors.blue,
          ),
        ),
        _tabRadio(
          title: "全部",
          value: RecordProcessStateEnum.All,
        ),
        _tabRadio(
          title: "待处理",
          value: RecordProcessStateEnum.Wait,
        ),
        _tabRadio(
          title: "已完成",
          value: RecordProcessStateEnum.Done,
        ),
        if (controller.userFeatures.any((e) => e.code == "RECGCZQX"))
          GestureDetector(
              onTap: () {
                Get.dialog(CreateExam(
                  onConfirm: () {
                    controller.getDatas();
                  },
                ));
              },
              child: Container(
                decoration: BoxDecoration(
                  color: Theme.of(context).primaryColor,
                  borderRadius: BorderRadius.circular(8),
                ),
                child: Row(
                  children: const [
                    Icon(
                      Icons.add,
                      color: Colors.white,
                      size: 24,
                    ),
                    if (kIsWeb) ...[
                      Text(
                        "新增",
                        style: TextStyle(fontSize: 20, color: Colors.white),
                      ),
                      SizedBox(
                        width: 5,
                      )
                    ]
                  ],
                ),
              ))
      ],
    );
  }

  Widget _buildRightContent() {
    return Column(
      children: [
        const SizedBox(height: 5),
        IconTabList(
          const {
            "报告查看": FIcons.report_a4,
          },
          selectedValue: [
            "报告查看",
          ][controller.selectedTabIndex],
          onPressed: (v) {
            controller.selectedTabIndex = v;
            controller.loadReports();
          },
        ),
        _buildRecordInfoView(),
      ],
    );
  }

  Widget _buildListView() {
    return GetBuilder<EcgListController>(
      id: "table",
      builder: (_) {
        return Column(
          children: [
            Expanded(
              child: ETable<ElectrocardiogramRecord>(
                autoHeight: false,
                noDataHintText: "暂无数据",
                columns: _buildTableColumns(),
                source: controller.residentList,
                loading: controller.tableLoading,
                onRowSelected: (value, index, idxs) {},
                onRowTap: (index) async {
                  controller.onRowTap(index);
                },
                cellPadding: const EdgeInsets.all(5),
                onAllRowsSelected: (value, idxs) => {},
                headerTextStyle: const TextStyle(
                  fontSize: 20,
                ),
              ),
            ),
            if (controller.currectSelected != -1) ...[
              _buildButtons(),
            ],
            const TablePagination(),
            const SizedBox(
              height: 16,
            )
          ],
        );
      },
    );
  }

  List<TableColumn<ElectrocardiogramRecord>> _buildTableColumns() {
    var textStyle = const TextStyle(
      fontSize: 16,
      overflow: TextOverflow.ellipsis,
    );
    return <TableColumn<ElectrocardiogramRecord>>[
      TableColumn<ElectrocardiogramRecord>(
        headerText: "编号",
        flex: 5,
        render: (rowData, index) => Container(
          padding: const EdgeInsets.symmetric(vertical: 12),
          child: Center(
            child: Text(
              rowData.physicalExamNumber ?? '',
              style: textStyle,
            ),
          ),
        ),
      ),
      TableColumn<ElectrocardiogramRecord>(
        headerText: "姓名",
        flex: 3,
        render: (rowData, index) => Center(
          child: Text(
            rowData.patientName ?? '',
            style: textStyle,
          ),
        ),
      ),
      TableColumn<ElectrocardiogramRecord>(
        headerText: "性别",
        flex: 2,
        render: (rowData, index) => Center(
          child: Text(
            patientGenderConvertAge(rowData.patientGender),
            style: textStyle,
          ),
        ),
      ),
      TableColumn<ElectrocardiogramRecord>(
        headerText: "年龄",
        flex: 2,
        render: (rowData, index) => Center(
          child: Text(
            birthDayConvertAge(rowData.birthday),
            style: textStyle,
          ),
        ),
      ),
      TableColumn<ElectrocardiogramRecord>(
        headerText: "所属医院",
        flex: 8,
        render: (rowData, index) => Center(
          child: Text(
            rowData.organizationName ?? "",
            style: textStyle,
          ),
        ),
      ),
      TableColumn<ElectrocardiogramRecord>(
        headerText: "状态",
        flex: 2,
        render: (rowData, index) => Center(
          child: Text(
            birthStateConvert(rowData.examState),
            style: textStyle,
          ),
        ),
      ),
      TableColumn<ElectrocardiogramRecord>(
        headerText: "检查时间",
        flex: 4,
        render: (rowData, index) => Center(
          child: Center(
            child: Text(
              DateFormat("yyy-MM-dd").format(
                rowData.createTime?.toLocal() ?? DateTime.now(),
              ),
              style: textStyle,
            ),
          ),
        ),
      ),
    ];
  }

  String birthDayConvertAge(DateTime? birthday) {
    if (birthday == null) {
      return "";
    }
    // 获取当前日期
    DateTime now = DateTime.now();

    // 计算年龄
    int age = calculateAge(birthday, now);

    return age.toString();
  }

  int calculateAge(DateTime birthDate, DateTime now) {
    // 计算年份差
    int age = now.year - birthDate.year;

    // 检查生日是否已经过了当前年份
    if (now.month < birthDate.month ||
        (now.month == birthDate.month && now.day < birthDate.day)) {
      age--;
    }

    return age;
  }

  String patientGenderConvertAge(GenderEnum patientGender) {
    switch (patientGender) {
      case GenderEnum.Male:
        return "男";
      case GenderEnum.Female:
        return "女";
      default:
        return "";
    }
  }

  Widget _buildRecordInfoView() {
    if (controller.reports.isEmpty) {
      return const NoDataView();
    }
    return Expanded(
      child: ListView(
        children: controller.reports
            .map(
              (e) => ReportRecord(
                currReport: e,
                onEditReport: (v) {
                  ElectrocardiogramRecord record =
                      controller.residentList[controller.currectSelected];
                  onEditReport.call(v, record);
                },
              ),
            )
            .toList(),
      ),
    );
  }

  Widget _buildButtons() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        SizedBox(
          width: 120,
          child: ElevatedButton(
            onPressed: () {
              ElectrocardiogramRecord examRecord =
                  controller.residentList[controller.currectSelected];
              onEditReport.call('', examRecord);
            },
            child: const Text("新建报告"),
          ),
        ),
        const SizedBox(
          width: 15,
        ),
        if (!kIsWeb)
          SizedBox(
            width: 120,
            child: ElevatedButton(
              onPressed: () {
                ElectrocardiogramRecord examRecord =
                    controller.residentList[controller.currectSelected];
                var physicalExamNumber = examRecord.physicalExamNumber ?? '';
                onExam.call(examRecord.code ?? '', physicalExamNumber);
              },
              child: const Text("检查"),
            ),
          ),
        if (controller.userFeatures.any((e) => e.code == "RECGCZQX")) ...[
          const SizedBox(
            width: 15,
          ),
          SizedBox(
            width: 120,
            child: ElevatedButton(
              onPressed: () async {
                ElectrocardiogramRecord examRecord =
                    controller.residentList[controller.currectSelected];
                bool result = await RPCBridge.ins.rpc.vitalElectrocardiogram
                    .removeElectrocardiogramAsync(
                  GetElectrocardiogramRecordRequest(
                    token: RPCBridge.ins.token,
                    code: examRecord.code,
                  ),
                );
                if (result) {
                  EasyLoading.showToast("删除成功");
                  controller.getDatas();
                  controller.loadReports();
                }
              },
              child: const Text("删除记录"),
            ),
          ),
        ],
      ],
    );
  }

  String birthStateConvert(ExamStateEnum examState) {
    switch (examState) {
      case ExamStateEnum.Reported:
        return "已完成";
      default:
        return "待处理";
    }
  }

  Widget _tabRadio(
      {required String title, required RecordProcessStateEnum value}) {
    return InkWell(
      onTap: () {
        controller.changeTypeFilter(value);
      },
      child: FContainer(
        margin: const EdgeInsets.only(right: 15),
        child: FRow(
          children: [
            FRadio(
              value: value,
              groupValue: controller.typeFilter,
              onChanged: (v) {
                controller.changeTypeFilter(value);
              },
            ),
            FText(
              title,
              style: TextStyle(
                color: controller.typeFilter == value
                    ? const Color(0xff2c77e5)
                    : const Color(0xff4c4948),
              ),
            ),
          ],
        ),
      ),
    );
  }
}
