import 'dart:convert';

import 'package:ecg_list_view/rpc/rpc_bridge.dart';
import 'package:fis_common/index.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_ui/base_define/page.dart';
import 'package:fis_ui/index.dart';
import 'package:fis_ui/usual/hover_text_button.dart';
import 'package:fis_ui/values/paddings.dart';
import 'package:fis_ui/values/spacings.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';
import 'package:fis_common/js_plateform/js_platform.dart';
import 'package:http/http.dart' as http;
import 'package:universal_html/html.dart' as html;
import 'package:fis_lib_print/src/printing.dart';

class ReportRecord extends StatelessWidget {
  /// 当前报告的数据
  final ReportDTO currReport;

  ///病人名字
  final String? patientName;

  ///编辑报告
  final void Function(String)? onEditReport;

  ///编辑报告标签
  final void Function()? onEditReportTag;

  ///检查
  final void Function()? onExamTap;

  ///图像
  final void Function()? onImageTap;

  final bool isSmailCard;

  const ReportRecord({
    Key? key,
    required this.currReport,
    this.patientName,
    this.onEditReport,
    this.onExamTap,
    this.onEditReportTag,
    this.onImageTap,
    this.isSmailCard = false,
  }) : super(key: key);

  static const String C_REPORTFORMAT = ".pdf";
  static const String C_DEFAULTFILENAME = "report";

  Widget textButton(String textString, Function onPressed) {
    return OutlinedButton(
      onPressed: () => onPressed.call(),
      style: ButtonStyle(
        padding: MaterialStateProperty.all(
            EdgeInsets.symmetric(horizontal: isSmailCard ? 5 : 0)),
        fixedSize: isSmailCard
            ? MaterialStateProperty.all<Size>(
                Size(75, 25),
              )
            : null,
      ),
      child: Text(
        textString,
        style: const TextStyle(color: Colors.black),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    var margin = const EdgeInsets.only(right: 20, left: 20, top: 20);
    var color = Colors.white;
    var padding = const EdgeInsets.only(top: 5, left: 15);
    var boxShadow = BoxShadow(
      color: FColorHelper.mixColor(Colors.black12, Colors.black38, 70),
      offset: const Offset(3.0, 3.0), //阴影y轴偏移量
      blurRadius: 6, //阴影模糊程度
      spreadRadius: 0, //阴影扩散程度
    );
    final child = Container(
      margin: margin,
      padding: padding,
      decoration: BoxDecoration(
        color: color,
        borderRadius: const BorderRadius.all(
          Radius.circular(8),
        ),
        boxShadow: [
          boxShadow,
        ],
      ),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          _buildRecordContent(context),
          Container(
            margin: const EdgeInsets.only(right: 5, top: 5),
            decoration: const BoxDecoration(
              border: Border(
                top: BorderSide(
                  color: Color.fromRGBO(216, 216, 216, 1),
                  width: 0.5,
                ),
              ),
            ),
            child: _buildSmailCardButtons(),
          ),
        ],
      ),
    );

    return child;
  }

  String? _generateNailUrl(ReportDTO reportList) {
    if (reportList.reportPreviewList == null ||
        reportList.reportPreviewList!.isEmpty) {
      return null;
    }
    return reportList.reportPreviewList!.first.thumbnailToken;
  }

  FWidget _generateReportIcon(String? reportNailUrl) {
    return const FCenter(
      child: FIcon(
        FIcons.report_a4,
        size: 70,
      ),
    );
  }

  Widget _buildRecordContent(BuildContext context) {
    String reportJsonName = patientName ?? '';
    if (reportJsonName.isEmpty) {
      reportJsonName = "未填写";
    }

    String reportUserName = currReport.reportUserName ?? '';
    if (reportUserName.isEmpty) {
      final labels = currReport.diagnosisLabels ?? [];
      reportUserName =
          labels.isNotEmpty ? currReport.diagnosisLabels![0] : "未填写";
    }
    return Row(
      mainAxisSize: MainAxisSize.min,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        Container(
          margin: const EdgeInsets.only(
            bottom: 15,
          ),
          child: _generateReportIcon(_generateNailUrl(currReport)),
        ),
        const SizedBox(width: 15),
        Expanded(
          child: Column(
            mainAxisSize: MainAxisSize.min,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Row(
                children: [
                  Container(
                    margin: const EdgeInsets.only(
                      right: 15,
                    ),
                    child: Text(dateTimeConvert(currReport.updateTime)),
                  ),
                ],
              ),
              const SizedBox(
                height: 5,
              ),
              const Text(
                "云端报告",
              ),
              const SizedBox(
                height: 5,
              ),
              Row(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  const Text("报告人:"),
                  const SizedBox(width: 5),
                  Text(
                    reportUserName,
                  ),
                ],
              ),
            ],
          ),
        ),
      ],
    );
  }

  String dateTimeConvert(DateTime? dateTime) {
    if (dateTime == null) {
      return "";
    }
    DateFormat formatter = DateFormat('yyyy-MM-dd HH:mm');
    String formattedDate = formatter.format(dateTime.toLocal());
    return formattedDate;
  }

  Widget _buildSmailCardButtons() {
    return Row(
      children: [
        if (currReport.canEditReport)
          textButton('修改', () {
            onEditReport?.call(currReport.reportCode ?? '');
          }),
        const SizedBox(
          width: 15,
        ),
        textButton('打印', () async {
          if (currReport.reportPreviewList?.any(
                  (element) => element.fileType == UploadFileTypeEnum.PDF) ??
              false) {
            var url = currReport.reportPreviewList
                ?.firstWhere(
                    (element) => element.fileType == UploadFileTypeEnum.PDF)
                .fileToken;
            var bytes = await downloadData(url!);
            if (kIsWeb) {
              var base64 = base64Encode(bytes);
              JSPlateForm.printPdf(base64);
            } else {
              await Printing.layoutPdf(
                onLayout: (format) async => bytes,
              );
            }
          } else {
            print("获取打印pdf失败");
            return;
          }
        }),
        const SizedBox(
          width: 15,
        ),
        if (kIsWeb)
          textButton('导出', () async {
            if (currReport.reportPreviewList?.any(
                    (element) => element.fileType == UploadFileTypeEnum.PDF) ??
                false) {
              var url = currReport.reportPreviewList
                  ?.firstWhere(
                      (element) => element.fileType == UploadFileTypeEnum.PDF)
                  .fileToken;
              downloadPdf(url!);
            } else {
              print("获取pdf失败");
              return;
            }
          }),
      ],
    );
  }

  void downloadPdf(String url) {
    final anchor = html.AnchorElement(href: url)
      ..style.display = 'none'
      ..download = 'downloaded.pdf'; // 设置下载文件的默认名称

    // 将链接添加到 DOM 中，以便触发下载
    html.document.body!.append(anchor);

    // 模拟点击链接以开始下载
    anchor.click();

    // 下载完成后移除链接
    html.document.body!.children.remove(anchor);
  }

  Future<Uint8List> downloadData(String url) async {
    final response = await http.get(Uri.parse(url));
    if (response.statusCode == 200) {
      return response.bodyBytes; // 直接返回 Uint8List
    } else {
      throw Exception('Failed to load data: ${response.statusCode}');
    }
  }
}
