import 'package:ecg_list_view/rpc/rpc_bridge.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:get/get.dart';
import 'alert_dialog.dart';
import 'dialog_date.dart';
import 'dialog_input.dart';
import 'dialog_select.dart';
import 'id_card_helper.dart';
import 'input.dart';
import 'package:fis_common/extensions/string.dart';
import 'package:date_format/date_format.dart';

import 'registration_info_model.dart';

class CreateExam extends StatefulWidget {
  final void Function()? onConfirm;

  const CreateExam({
    super.key,
    this.onConfirm,
  });

  @override
  State<StatefulWidget> createState() {
    return CreateExamState();
  }
}

class CreateExamState extends State<CreateExam> {
  final RegistrationInfoModel patient = RegistrationInfoModel();
  final List<VSelectGenderEnumModel> selectGenderList = [
    VSelectGenderEnumModel(code: GenderEnum.Female, name: "女"),
    VSelectGenderEnumModel(code: GenderEnum.Male, name: "男"),
    VSelectGenderEnumModel(code: GenderEnum.Unknown, name: "未知的性别"),
    VSelectGenderEnumModel(code: GenderEnum.Unspecified, name: "未说明的性别"),
  ];
  final cardNoController = TextEditingController();
  final nameController = TextEditingController();
  final genderController = TextEditingController();
  final birthdayController = TextEditingController();
  final phoneController = TextEditingController();
  final addressController = TextEditingController();

  @override
  Widget build(BuildContext context) {
    return Alert(
      title: "新建检查",
      width: 600,
      content: _buildView(),
      onConfirm: () async {
        await RPCBridge.ins.rpc.vitalHealthExamBooking.addRegiterInfoAsync(
          AddRegiterInfoRequest(
            token: RPCBridge.ins.token,
            iDCardNo: patient.cardNo,
            name: patient.patientName,
            birthday: patient.birthday,
            sex: genderController.text,
            adress: patient.patientAddress,
            phone: patient.phone,
          ),
        );
        widget.onConfirm?.call();
        Get.back();
      },
    );
  }

  Widget _buildView() {
    return ListView(
      children: [
        _buildInputItem(
          "身份证号",
          value: patient.cardNo,
          controller: cardNoController,
          onTap: () async {
            var result = await DialogInput(
              initialValue: patient.cardNo,
              title: "身份证号",
              onConfirmVerification: (v) {
                bool isValid = IdCardHelper.validateIDCard(v);
                if (!isValid) {
                  EasyLoading.showToast("请填写正确的证件号");
                }
                return isValid;
              },
              showCancel: true,
            ).show();
            if (result == null) {
              return;
            }

            result = result.toString().toUpperCase();
            patient.cardNo = result;
            cardNoController.text = result;
            patient.birthday = _getBirthDayofIdNumber(result);
            birthdayController.text = getBirthDay(patient.birthday);
            patient.patientGender = _checkGenderFromID(result);
            var selectedData = selectGenderList.firstWhereOrNull(
                (element) => element.code == patient.patientGender);
            if (selectedData != null) {
              genderController.text = selectedData.name;
            }
          },
        ),
        _buildInputItem(
          "姓名",
          value: patient.patientName ?? '',
          controller: nameController,
          onTap: () async {
            var result = await DialogInput(
              initialValue: patient.patientName ?? '',
              title: "姓名",
            ).show();
            if (result != null) {
              patient.patientName = result;
              nameController.text = result;
            }
          },
        ),
        _buildInputItem(
          "性别",
          controller: genderController,
          value: selectGenderList
              .firstWhereOrNull(
                  (element) => element.code == patient.patientGender)
              ?.name,
          onTap: () async {
            String? result = await DialogSelect<VSelectGenderEnumModel, String>(
              source: selectGenderList,
              labelGetter: (data) => data.name,
              valueGetter: (data) => data.code.index.toString(),
              initialValue: patient.patientGender?.index.toString(),
              title: "性别",
            ).show();
            if (result != null) {
              patient.patientGender = GenderEnum.values[int.parse(result)];
              var selectedData = selectGenderList.firstWhereOrNull(
                  (element) => element.code == patient.patientGender);
              if (selectedData != null) {
                genderController.text = selectedData.name;
              }
            }
          },
          suffixIcon: const Icon(
            Icons.chevron_right_rounded,
            size: 30,
          ),
        ),
        _buildInputItem(
          "生日",
          value: getBirthDay(patient.birthday),
          controller: birthdayController,
          onTap: () async {
            DateTime? result;
            if (kIsWeb) {
              result = await showDatePicker(
                context: Get.context!,
                initialDate: patient.birthday ?? DateTime.now(),
                firstDate: DateTime(1900),
                lastDate: DateTime(2100),
              );
            } else {
              result = await DateDialog(
                initialValue: patient.birthday,
                title: "生日",
              ).show();
            }
            if (result != null) {
              patient.birthday = result;
              birthdayController.text = getBirthDay(result);
            }
          },
          suffixIcon: const Icon(
            Icons.chevron_right_rounded,
            size: 30,
          ),
        ),
        _buildInputItem(
          "手机号",
          controller: phoneController,
          value: patient.phone,
          onTap: () async {
            var result = await DialogInput(
              initialValue: patient.phone ?? '',
              title: "手机号",
              showCancel: true,
              onConfirmVerification: (v) {
                var isValid = (v.length == 11 && IdCardHelper.isNumeric(v));
                if (!isValid) {
                  EasyLoading.showToast("请填写正确的手机号");
                }
                return isValid;
              },
            ).show();
            if (result != null) {
              patient.phone = result;
              phoneController.text = result;
            }
          },
        ),
        _buildInputItem(
          "家庭住址",
          controller: addressController,
          value: patient.patientAddress ?? '',
          onTap: () async {
            var result = await DialogInput(
              initialValue: patient.patientAddress ?? '',
              title: "家庭住址",
            ).show();
            if (result != null) {
              patient.patientAddress = result;
              addressController.text = result;
            }
          },
        ),
      ],
    );
  }

  String getBirthDay(DateTime? birthDay) {
    if (birthDay == null) {
      return "";
    }
    return formatDate(birthDay, [yyyy, "-", mm, "-", dd]);
  }

  DateTime? _getBirthDayofIdNumber(String idCardNumber) {
    if (idCardNumber.isEmpty) return DateTime(1970, 1, 1);
    final idCardRegex = RegExp(r'^(\d{6})(\d{4})(\d{2})(\d{2})(\d{3})(\d|X)$');
    final match = idCardRegex.firstMatch(idCardNumber);
    if (match != null) {
      final year = int.parse(match.group(2)!);
      final month = int.parse(match.group(3)!);
      final day = int.parse(match.group(4)!);

      final birthDate = DateTime(year, month, day);

      return birthDate;
    }
    return null; // 返回一个默认值，表示无法提取出生日期
  }

  GenderEnum _checkGenderFromID(String idNumber) {
    if (idNumber.isNullOrEmpty) {
      return GenderEnum.Unspecified;
    }
    int secondLastDigit = int.parse(idNumber[idNumber.length - 2]);

    if (secondLastDigit % 2 == 0) {
      return GenderEnum.Female;
    } else {
      return GenderEnum.Male;
    }
  }

  Widget _buildInputItem(
    String title, {
    String? value,
    Function()? onTap,
    Widget? suffixIcon,
    int? maxLines,
    TextEditingController? controller,
  }) {
    return Container(
      padding: const EdgeInsets.all(10),
      child: Row(
        children: [
          Container(
            width: 150,
            padding: const EdgeInsets.symmetric(vertical: 8),
            child: Text(
              title,
              style: const TextStyle(fontSize: 20),
            ),
          ),
          Expanded(
            child: Input(
              initialValue: value,
              radius: 4,
              readOnly: true,
              suffixIcon: suffixIcon,
              maxLines: maxLines,
              controller: controller,
              onTap: () {
                onTap?.call();
              },
            ),
          )
        ],
      ),
    );
  }
}
