import 'package:ecg_list_view/ecg_list/controller.dart';
import 'package:ecg_list_view/rpc/rpc_bridge.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';

import 'alert_dialog.dart';
import 'dialog_date.dart';
import 'input.dart';

// ignore: must_be_immutable
class FilterView extends StatefulWidget {
  final void Function(DateTime, DateTime) onConfirm;
  final DateTime? startTime;
  final DateTime? endTime;
  final bool isExistSubordinate;

  const FilterView({
    required this.onConfirm,
    this.startTime,
    this.endTime,
    this.isExistSubordinate = false,
    super.key,
  });

  @override
  State<StatefulWidget> createState() {
    return FilterViewState();
  }
}

class FilterViewState extends State<FilterView> {
  DateTime _startTime = DateTime.now();
  DateTime _endTime = DateTime.now();
  final controller = Get.find<EcgListController>();
  final _startController = TextEditingController();
  final _endController = TextEditingController();

  FilterViewState() {
    var now = DateTime.now();
    DateTime newDay = DateTime(now.year, now.month, now.day);
    _startTime = newDay;
    _endTime = newDay.add(const Duration(hours: 23, minutes: 59));
  }

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    // 设置日期格式为 年-月-日 时:分
    String startFormattedDate =
        DateFormat('yyyy-MM-dd HH:mm').format(widget.startTime ?? _startTime);
    String endFormattedDate =
        DateFormat('yyyy-MM-dd HH:mm').format(widget.endTime ?? _endTime);
    _startController.text = startFormattedDate;
    _endController.text = endFormattedDate;
    return Alert(
      title: "筛选时间",
      width: 600,
      onConfirm: () {
        widget.onConfirm.call(_startTime, _endTime);
        Get.back();
      },
      content: Container(
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.circular(8.0), // 设置圆角的半径
        ),
        width: 500,
        height: 220,
        child: Column(
          children: [
            _buildInputItem(
              "开始时间",
              value: startFormattedDate,
              onTap: _onChangeStartTime,
              suffixIcon: const Icon(
                Icons.chevron_right_rounded,
                size: 30,
              ),
              controller: _startController,
            ),
            _buildInputItem(
              "结束时间",
              value: endFormattedDate,
              onTap: _onChangeEndTime,
              suffixIcon: const Icon(
                Icons.chevron_right_rounded,
                size: 30,
              ),
              controller: _endController,
            ),
            if (widget.isExistSubordinate) _buildLowerDatasSwitch(),
            const Expanded(child: SizedBox()),
          ],
        ),
      ),
    );
  }

  Widget _buildLowerDatasSwitch() {
    return Row(
      children: [
        const SizedBox(
          width: 20,
        ),
        const Text(
          "包含下级数据",
          style: TextStyle(fontSize: 22),
        ),
        const SizedBox(
          width: 10,
        ),
        Switch(
          value: controller.showLowerDatas,
          onChanged: (v) {
            controller.changeDatas(v);
            setState(() {});
          },
        )
      ],
    );
  }

  /// 构建表格输入框
  Widget _buildInputItem(
    String title, {
    String? value,
    Function()? onTap,
    Widget? suffixIcon,
    int? maxLines,
    void Function(String)? onChanged,
    TextEditingController? controller,
  }) {
    return Container(
      padding: const EdgeInsets.only(
        left: 20,
        right: 20,
      ),
      child: Row(
        children: [
          Container(
            width: 150,
            padding: const EdgeInsets.symmetric(vertical: 16),
            child: Text(
              title,
              style: const TextStyle(fontSize: 22),
            ),
          ),
          Expanded(
            child: Input(
              initialValue: value,
              radius: 4,
              readOnly: onTap == null ? false : true,
              suffixIcon: suffixIcon,
              maxLines: maxLines,
              onChanged: onChanged,
              controller: controller,
              onTap: () {
                onTap?.call();
              },
            ),
          )
        ],
      ),
    );
  }

  void _onChangeStartTime() async {
    DateTime now = DateTime.now();
    DateTime? result;
    var initialValue = widget.startTime ?? _startTime;
    if (kIsWeb) {
      result = await showDatePicker(
        context: Get.context!,
        initialDate: initialValue,
        firstDate: DateTime(1900),
        lastDate: DateTime(2100),
      );
      if (result == null) {
        return;
      }
      var time = await showTimePicker(
        context: Get.context!,
        initialTime: const TimeOfDay(hour: 9, minute: 0),
      );
      if (time != null) {
        result = result.add(Duration(hours: time.hour, minutes: time.minute));
      }
    } else {
      result = await DateDialog(
        maxValue: _endTime,
        title: '开始时间',
        initialValue: initialValue,
      ).show();
      if (result == null) {
        return;
      }
      var time = await VDialogDateTime2(
        minimumDate: now,
        maxValue: _endTime,
        title: '开始时间',
        initialValue: initialValue,
      ).show();
      if (time != null) {
        result = result.add(Duration(hours: time.hour, minutes: time.minute));
      }
    }
    String startFormattedDate = DateFormat('yyyy-MM-dd HH:mm').format(result);
    _startTime = result;
    _startController.text = startFormattedDate;
  }

  Future<void> _onChangeEndTime() async {
    DateTime now = DateTime.now();
    DateTime? result;
    var initialValue = widget.endTime ?? _endTime;
    if (kIsWeb) {
      result = await showDatePicker(
        context: Get.context!,
        initialDate: initialValue,
        firstDate: DateTime(1900),
        lastDate: DateTime(2100),
      );
      if (result == null) {
        return;
      }
      var time = await showTimePicker(
        context: Get.context!,
        initialTime: const TimeOfDay(hour: 9, minute: 0),
      );
      if (time != null) {
        result = result.add(Duration(hours: time.hour, minutes: time.minute));
      }
    } else {
      result = await DateDialog(
        maxValue: now,
        minimumDate: _startTime,
        title: '结束时间',
        initialValue: initialValue,
      ).show();
      if (result == null) {
        return;
      }
      var time = await VDialogDateTime2(
        minimumDate: _startTime,
        title: '结束时间',
        initialValue: initialValue,
      ).show();
      if (time != null) {
        result = result.add(Duration(hours: time.hour, minutes: time.minute));
      }
    }
    _endTime = result;
    String endFormattedDate = DateFormat('yyyy-MM-dd HH:mm').format(result);
    _endController.text = endFormattedDate;
  }
}
